from typing import Optional

from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from flask import jsonify

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.exception.service_error_code_enum import ServiceErrorCode
from bot_api.exception.service_exception import ServiceException
from bot_api.filter.discord.server_select_criteria import ServerSelectCriteria
from bot_api.model.discord.server_dto import ServerDTO
from bot_api.model.discord.server_filtered_result_dto import ServerFilteredResultDTO
from bot_api.model.error_dto import ErrorDTO
from bot_api.transformer.server_transformer import ServerTransformer
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.auth_role_enum import AuthRoleEnum
from bot_data.model.server import Server


class DiscordService:
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        auth: AuthServiceABC,
        auth_users: AuthUserRepositoryABC,
        users: UserRepositoryABC,
    ):
        self._bot = bot
        self._servers = servers
        self._auth = auth
        self._auth_users = auth_users
        self._users = users

    def _to_dto(self, x: Server) -> Optional[ServerDTO]:
        guild = self._bot.get_guild(x.discord_server_id)
        if guild is None:
            return ServerTransformer.to_dto(x, "", 0, None)

        return ServerTransformer.to_dto(x, guild.name, guild.member_count, guild.icon)

    async def get_all_servers(self) -> List[ServerDTO]:
        servers = List(ServerDTO, self._servers.get_servers())
        return servers.select(self._to_dto).where(lambda x: x.name != "")

    async def get_all_servers_by_user(self) -> List[ServerDTO]:
        token = self._auth.get_decoded_token_from_request()
        if token is None or "email" not in token or "role" not in token:
            raise ServiceException(ServiceErrorCode.InvalidData, "Token invalid")

        role = AuthRoleEnum(token["role"])
        servers = self._servers.get_servers()
        if role != AuthRoleEnum.admin:
            auth_user = self._auth_users.find_auth_user_by_email(token["email"])
            if auth_user is not None:
                user_ids = auth_user.users.select(lambda x: x.server is not None and x.server.server_id)
                servers = servers.where(lambda x: x.server_id in user_ids)

        servers = List(ServerDTO, servers)
        return servers.select(self._to_dto).where(lambda x: x.name != "")

    async def get_filtered_servers_async(self, criteria: ServerSelectCriteria) -> ServerFilteredResultDTO:
        token = self._auth.get_decoded_token_from_request()
        if token is None or "email" not in token or "role" not in token:
            raise ServiceException(ServiceErrorCode.InvalidData, "Token invalid")

        role = AuthRoleEnum(token["role"])
        filtered_result = self._servers.get_filtered_servers(criteria)
        # filter out servers, where the user not exists
        if role != AuthRoleEnum.admin:
            auth_user = self._auth_users.find_auth_user_by_email(token["email"])
            if auth_user is not None:
                user_ids = auth_user.users.select(lambda x: x.server is not None and x.server.server_id)
                filtered_result.result = filtered_result.result.where(lambda x: x.server_id in user_ids)

        servers: List = filtered_result.result.select(self._to_dto).where(lambda x: x.name != "")
        result = List(ServerDTO, servers)

        if criteria.name is not None and criteria.name != "":
            result = result.where(
                lambda x: criteria.name.lower() in x.name.lower() or x.name.lower() == criteria.name.lower()
            )

        return ServerFilteredResultDTO(List(ServerDTO, result), servers.count())

    async def get_server_by_id_async(self, id: int) -> ServerDTO:
        server = self._servers.get_server_by_id(id)
        guild = self._bot.get_guild(server.discord_server_id)

        server_dto = ServerTransformer.to_dto(server, guild.name, guild.member_count, guild.icon)
        return server_dto
