from datetime import datetime
from typing import Union

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnMemberJoinABC
from cpl_translation import TranslatePipe

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.known_user import KnownUser
from bot_data.model.user import User
from bot_data.model.user_joined_server import UserJoinedServer
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class BaseOnMemberJoinEvent(OnMemberJoinABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        base_helper: BaseHelperABC,
        messenger: MessageServiceABC,
        permissions: PermissionServiceABC,
        db: DatabaseContextABC,
        known_users: KnownUserRepositoryABC,
        users: UserRepositoryABC,
        servers: ServerRepositoryABC,
        user_joins: UserJoinedServerRepositoryABC,
        translate: TranslatePipe,
    ):
        OnMemberJoinABC.__init__(self)
        self._config = config
        self._logger = logger
        self._base_helper = base_helper
        self._messenger = messenger
        self._permission_service = permissions
        self._db = db
        self._known_users = known_users
        self._users = users
        self._servers = servers
        self._user_joins = user_joins
        self._t = translate

    def _check_for_known_user(self, member: Union[discord.User, discord.Member]):
        self._logger.debug(__name__, f"Check if user is already known {member}")
        try:
            user = self._known_users.find_user_by_discord_id(member.id)
            if user is not None:
                return

            self._logger.debug(__name__, f"Add user: {member.id}")
            self._known_users.add_user(KnownUser(member.id))
            self._db.save_changes()
        except Exception as e:
            self._logger.error(__name__, f"Cannot get user {member.id}", e)

    async def _add_if_not_exists_user_async(self, member: Union[discord.User, discord.Member]):
        self._logger.debug(__name__, f"Check if user exists {member}")
        settings: BaseServerSettings = self._base_helper.get_config(member.guild.id)
        await self._messenger.send_dm_message(
            self._t.transform("modules.base.welcome_message").format(member.guild.name),
            member,
        )

        for admin in self._permission_service.get_admins(member.guild.id):
            await self._messenger.send_dm_message(
                self._t.transform("modules.base.welcome_message_for_team").format(member.mention),
                admin,
            )

        for moderator in self._permission_service.get_moderators(member.guild.id):
            await self._messenger.send_dm_message(
                self._t.transform("modules.base.welcome_message_for_team").format(member.mention),
                moderator,
            )

        try:
            server = self._servers.get_server_by_discord_id(member.guild.id)
            user = self._users.find_user_by_discord_id_and_server_id(member.id, server.server_id)
            if user is not None:
                self._user_joins.add_user_joined_server(UserJoinedServer(user, datetime.now()))
                self._db.save_changes()
                return

            self._logger.debug(__name__, f"Add user: {member.id}")
            self._users.add_user(User(member.id, 0, server))
            self._db.save_changes()
            user = self._users.get_user_by_discord_id_and_server_id(member.id, server.server_id)
            self._user_joins.add_user_joined_server(UserJoinedServer(user, datetime.now()))
            self._db.save_changes()
        except Exception as e:
            self._logger.error(__name__, f"Cannot get user {member.id}", e)

    @EventChecks.check_is_ready()
    async def on_member_join(self, member: discord.Member):
        self._logger.debug(__name__, f"Module {type(self)} started")
        self._check_for_known_user(member)
        await self._add_if_not_exists_user_async(member)
